unit DataTable;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Prototype, Grids, BaseGrid, AdvGrid, xmldom, XMLIntf, msxmldom, XMLDoc;

type
  TfrmDataTable = class(TfrmPrototype)
    sgTable: TAdvStringGrid;
    procedure sgTableCanSort(Sender: TObject; ACol: Integer;
      var DoSort: Boolean);
    procedure sgTableColumnMove(Sender: TObject; ACol: Integer;
      var Allow: Boolean);
    procedure sgTableColumnMoved(Sender: TObject; FromIndex,
      ToIndex: Integer);
  private
    { Private declarations }
    fMoving : Boolean;
    procedure LoadColumns(xmlColumns : TXMLDocument);
    procedure LoadData(xmlData : TXMLDocument);
  public
    { Public declarations }
    constructor Create(AOnwer : TComponent; xmlOptions : TXMLDocument); overload;
  end;

var
  frmDataTable: TfrmDataTable;

implementation

{$R *.dfm}

{ TfrmDataTable }

constructor TfrmDataTable.Create(AOnwer: TComponent;
  xmlOptions: TXMLDocument);
var
  nodeOptions : IXMLNode;
  xmlColumns, xmlData : TXMLDocument;
begin
  inherited;
  fMoving := false;
  nodeOptions := xmlOptions.ChildNodes.FindNode('options');
  xmlColumns := TXMLDocument.Create(Self);
  xmlColumns.FileName :=  nodeOptions.Attributes['schema'];
  xmlColumns.Active := true;
  xmlData := TXMLDocument.Create(Self);
  xmlData.FileName := nodeOptions.Attributes['sampledata'];
  xmlData.Active := true;
  LoadColumns(xmlColumns);
  LoadData(xmlData);
  if nodeOptions.Attributes['allowsort'] = 'true' then
    sgTable.SortSettings.Show := true;
end;

procedure TfrmDataTable.LoadColumns(xmlColumns: TXMLDocument);
var
  nodeComplexType, nodeCurElement : IXMLNode;
  i : Integer;
begin
  nodeComplexType := xmlColumns.ChildNodes.FindNode('complexType');
  sgTable.ColCount := nodeComplexType.ChildNodes.Count;
  for i:=0 to nodeComplexType.ChildNodes.Count-1 do
  begin
    nodeCurElement := nodeComplexType.ChildNodes.Get(i);
    sgTable.Cells[i,0] := nodeCurElement.Attributes['name'];
  end;
end;

procedure TfrmDataTable.LoadData(xmlData: TXMLDocument);
var
  nodeDataElements, nodeCurRow, nodeCurColumn : IXMLNode;
  i,j : Integer;
begin
  nodeDataElements := xmlData.ChildNodes.FindNode('elements');
  sgTable.RowCount := nodeDataElements.ChildNodes.Count + 1;
  for i:=0 to nodeDataElements.ChildNodes.Count-1 do
  begin
    nodeCurRow := nodeDataElements.ChildNodes[i];
    for j:=0 to nodeCurRow.ChildNodes.Count-1 do
    begin
      nodeCurColumn := nodeCurRow.ChildNodes[j];
      sgTable.Cells[j,i+1] := nodeCurColumn.NodeValue;
    end;
  end;
  sgTable.AutoSize := true;
end;

procedure TfrmDataTable.sgTableCanSort(Sender: TObject; ACol: Integer;
  var DoSort: Boolean);
begin
  inherited;
  DoSort := true;
  if sckPrototype.Active then
    sckPrototype.Socket.SendText('Sort By ' + sgTable.Cells[ACol,0]);
end;

procedure TfrmDataTable.sgTableColumnMove(Sender: TObject; ACol: Integer;
  var Allow: Boolean);
begin
  inherited;
  if sckPrototype.Active AND fMoving = false then
  begin
    sckPrototype.Socket.SendText('Move ' + sgTable.Cells[ACol,0] + ' Col');
    fMoving := true;
  end;
  Allow := true;
end;

procedure TfrmDataTable.sgTableColumnMoved(Sender: TObject; FromIndex,
  ToIndex: Integer);
begin
  inherited;
  if sckPrototype.Active then
  begin
    sckPrototype.Socket.SendText('Select New Location');
    fMoving := false;
  end;
end;

end.
