unit ETS;

interface

uses  SysUtils, CTT;

type
  EOutOfRange = class(Exception);
  EImportError = class(Exception);

  TState = class;
  TStateTransition = class;
  
  TEnabledTaskSet = class
    private
      fEnabledTasks : array of TCTTNode;
      fTaskCount : Integer;
    function GetTask(Index: Integer): TCTTNode;
    procedure SetTask(Index: Integer; const Value: TCTTNode);
    public
      property EnabledTasks[Index : Integer] : TCTTNode
            read GetTask write SetTask; default;
      property Count : Integer
            read fTaskCount;
      procedure AddTask(const Task : TCTTNode);
      destructor Destroy; override;
      constructor Create; overload;
  end;

{  TEnabledTaskCollection = class
    private
      fEnabledTaskSets : array of TEnabledTaskSet;
      fTaskSetCount : Integer;
      fInitialTaskSet: Integer;
    function GetEnabledTaskSet(Index: Integer): TEnabledTaskSet;
    procedure SetEnabledTaskSet(Index: Integer;
      const Value: TEnabledTaskSet);
    public
      property EnabledTaskSets[Index : Integer] : TEnabledTaskSet
            read GetEnabledTaskSet write SetEnabledTaskSet; default;
      property InitialTaskSet : Integer
            read fInitialTaskSet write fInitialTaskSet;
      property Count : Integer
            read fTaskSetCount write fTaskSetCount;
      procedure AddTaskSet(const TaskSet : TEnabledTaskSet);
      destructor Destroy; override;
      constructor Create; overload;
  end;         }

  TState = class
    private
      fEnabledTaskSet : TEnabledTaskSet;
      fTransitionCount : Integer;
      fStateTransitions : array of TStateTransition;
    function GetTranstion(Index: Integer): TStateTransition;
    procedure SetTranstion(Index: Integer; const Value: TStateTransition);
    public
      property Transitions[Index : Integer] : TStateTransition
            read GetTranstion write SetTranstion; default;
      property EnabledTaskSet : TEnabledTaskSet
            read fEnabledTaskSet write fEnabledTaskSet;
      property TransitionCount : Integer
            read fTransitionCount write fTransitionCount;
      procedure AddTransition(Transition : TStateTransition);
      constructor Create;
      destructor Destroy; override;
  end;

  TAcceptState = class(TState);

  TStateTransition = class
    private
      fDestinationState : TState;
      fTransitionTask : TCTTNode;
    public
      property TransitionTask : TCTTNode
            read fTransitionTask write fTransitionTask;
      property DestinationState : TState
            read fDestinationState write fDestinationState;
      constructor Create; overload;
  end;

  TStateTransitionNetwork = class
    private
      fInitialStateIndex : Integer;
      fStates : array of TState;
      fStateCount : Integer;
    function GetState(Index: Integer): TState;
    procedure SetState(Index: Integer; const Value: TState);
    public
      property InitialStateIndex : Integer
            read fInitialStateIndex write fInitialStateIndex;
      property States[Index : Integer] : TState
            read GetState write SetState; default;
      procedure AddState(State : TState);
      function NextState(InitialState : TState; TaskPerformed  : TCTTNode) : TState;
      constructor Create; overload;
      destructor Destroy; override;
  end;



  {Functions to import EnabledTaskSets that have been output from TaskLib
    http://www.edm.luc.ac.be/software/TaskLib/}
  function ImportSTN(const FileName : String; cttRootNode : TCTTNode) : TStateTransitionNetwork;

implementation
{Import Functions}

function ImportSTN(const FileName : String; cttRootNode : TCTTNode) : TStateTransitionNetwork;
var
  fileETC : TextFile;
  sCurLine, sCurTaskName : String;
  etsCurTaskSet : TEnabledTaskSet;
  stateCurState : TState;
  stCurTransition : TStateTransition;
  cttCurTask : TCTTNode;
  iPosOpen, iPosClose : Integer;
  iOriginStateIndex, iDestStateIndex : Integer;
begin

  Result := TStateTransitionNetwork.Create;

  try
    Assign(fileETC, FileName);
    Reset(fileETC);
  except
    Exit;
  end;

  sCurLine := ' ';
  while Pos('=', sCurLine)=0 do
    ReadLn(fileETC, sCurLine);

  ReadLn(fileETC, sCurLine);
  //For each of the Enabled Task Sets
  while not (sCurLine = '') do
  begin

    etsCurTaskSet := TEnabledTaskSet.Create;
    stateCurState := TState.Create;
    stateCurState.EnabledTaskSet := etsCurTaskSet;

    iPosOpen := Pos('{', sCurLine);
    iPosClose := Pos('}',sCurLine);
    sCurLine := Copy(sCurLine, iPosOpen + 1, iPosClose-iPosOpen-1);
    sCurLine := sCurLine + ', ';
    //For each of the Tasks in the set.
    while Pos(',', sCurLine)>0 do
    begin
      sCurTaskName := Trim(Copy(sCurLine, 1, Pos(',',sCurLine)-1));
      cttCurTask := cttRootNode.FindChildByName(sCurTaskName);
      if cttCurTask = nil then
        raise EImportError.Create('Cannot import enabled task set.'#13#10'Task ('
                                  + sCurTaskName + ') not found')
      else
      begin
        etsCurTaskSet.AddTask(cttCurTask);
        sCurLine := Copy(sCurLine, Pos(',',sCurLine) + 1, Length(sCurLine) );
      end;
    end;
    Result.AddState(stateCurState);
    ReadLn(fileETC, sCurLine);
  end;

  while Pos('=', sCurLine)=0 do
    ReadLn(fileETC, sCurLine);

  ReadLn(fileETC, sCurLine);
  sCurLine := Trim(sCurLine);

  sCurLine := Copy(sCurLine, Pos(' ', sCurLine) + 1, Length(sCurLine));
  Result.InitialStateIndex := StrToInt(sCurLine);

  while Pos('=', sCurLine)=0 do
    ReadLn(fileETC, sCurLine);

  ReadLn(fileETC, sCurLine);

  while not SeekEof(fileETC) do
  begin
    sCurLine := Copy(sCurLine, Pos('ETS',sCurLine)+3, Length(sCurLine));
    sCurLine := Trim(sCurLine);
    iOriginStateIndex := StrToInt(Copy(sCurLine,1,Pos(' ', sCurLine)-1));
    sCurLine := Copy(sCurLine, Pos('->',sCurLine)+2, Length(sCurLine));
    sCurLine := Trim(sCurLine);
    sCurTaskName := Copy(sCurLine,1,Pos('->',sCurLine)-1);
    sCurTaskName := Trim(sCurTaskName);
    cttCurTask := cttRootNode.FindChildByName(sCurTaskName);
    stCurTransition := TStateTransition.Create;
    stCurTransition.TransitionTask := cttCurTask;
    if Pos('Accept State', sCurLine)>0 then
    begin
      stCurTransition.DestinationState := TAcceptState.Create;
    end
    else
    begin
      sCurLine := Copy(sCurLine, Pos('ETS',sCurLine)+3, Length(sCurLine));
      sCurLine := Trim(sCurLine);
      iDestStateIndex := StrToInt(sCurLine);
      stCurTransition.DestinationState := Result.States[iDestStateIndex];
    end;
    Result.States[iOriginStateIndex].AddTransition(stCurTransition);
    ReadLn(fileETC, sCurLine);
  end;

end;

{ TEnabledTaskSet }

procedure TEnabledTaskSet.AddTask(const Task: TCTTNode);
begin
    SetLength(fEnabledTasks, fTaskCount+1);
    fEnabledTasks[fTaskCount] := Task;
    inc(fTaskCount);
end;

constructor TEnabledTaskSet.Create;
begin
  SetLength(fEnabledTasks, 0);
  fTaskCount := 0;
end;

destructor TEnabledTaskSet.Destroy;
begin
  SetLength(fEnabledTasks, 0);
  fTaskCount := 0;
  inherited;
end;

function TEnabledTaskSet.GetTask(Index: Integer): TCTTNode;
begin
  if (Index>fTaskCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  Result := fEnabledTasks[Index];
end;

procedure TEnabledTaskSet.SetTask(Index: Integer; const Value: TCTTNode);
begin
  if (Index>fTaskCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  fEnabledTasks[Index] := Value;
end;

{ TEnabledTaskCollection }
   {
procedure TEnabledTaskCollection.AddTaskSet(
  const TaskSet: TEnabledTaskSet);
begin
  SetLength(fEnabledTaskSets, fTaskSetCount+1);
  fEnabledTaskSets[fTaskSetCount] := TaskSet;
  inc(fTaskSetCount);
end;

constructor TEnabledTaskCollection.Create;
begin
  SetLength(fEnabledTaskSets, 0);
  fTaskSetCount := 0;
  fInitialTaskSet := 0;
end;

destructor TEnabledTaskCollection.Destroy;
var
  i : Integer;
begin
  for i:=0 to fTaskSetCount-1 do
  begin
    fEnabledTaskSets[i].Destroy;
  end;

  SetLength(fEnabledTaskSets,0);
  fTaskSetCount := 0;
  inherited;
end;

function TEnabledTaskCollection.GetEnabledTaskSet(
  Index: Integer): TEnabledTaskSet;
begin
  if (Index>fTaskSetCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  Result := fEnabledTaskSets[Index];
end;

procedure TEnabledTaskCollection.SetEnabledTaskSet(Index: Integer;
  const Value: TEnabledTaskSet);
begin
  if (Index>fTaskSetCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  fEnabledTaskSets[Index] := Value;
end;                      }

{ TStateTransition }

constructor TStateTransition.Create;
begin
  fDestinationState := TState.Create;
  fTransitionTask := TCTTNode.Create;
end;

{ TState }

procedure TState.AddTransition(Transition: TStateTransition);
begin
  SetLength(fStateTransitions, fTransitionCount+1);
  fStateTransitions[fTransitionCount] := Transition;
  inc(fTransitionCount);
end;

constructor TState.Create;
begin
  SetLength(fStateTransitions, 0);
  fTransitionCount := 0;
  fEnabledTaskSet := TEnabledTaskSet.Create;
end;

destructor TState.Destroy;
var
  i : Integer;
begin
  for i:=0 to fTransitionCount-1 do
  begin
    fStateTransitions[i].Destroy;
  end;
  fEnabledTaskSet.Destroy;
  fTransitionCount := 0;
  inherited;
end;

function TState.GetTranstion(Index: Integer): TStateTransition;
begin
  if (Index>fTransitionCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  Result := fStateTransitions[Index];
end;

procedure TState.SetTranstion(Index: Integer;
  const Value: TStateTransition);
begin
  if (Index>fTransitionCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  fStateTransitions[Index] := Value;
end;

{ TStateTransitionNetwork }

procedure TStateTransitionNetwork.AddState(State: TState);
begin
  SetLength(fStates, fStateCount+1);
  fStates[fStateCount] := State;
  inc(fStateCount);
end;

constructor TStateTransitionNetwork.Create;
begin
  SetLength(fStates, 0);
  fStateCount := 0;
  fInitialStateIndex := -1;
end;

destructor TStateTransitionNetwork.Destroy;
begin

  inherited;
end;

function TStateTransitionNetwork.GetState(Index: Integer): TState;
begin
  if (Index>fStateCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  Result := fStates[Index];
end;

function TStateTransitionNetwork.NextState(InitialState: TState;
  TaskPerformed: TCTTNode): TState;
var
  i : Integer;
begin
  for i:=0 to InitialState.TransitionCount-1 do
  begin
    if InitialState.Transitions[i].fTransitionTask = TaskPerformed then
    begin
      Result := InitialState.Transitions[i].DestinationState;
      Exit;
    end;
  end;
  Result := InitialState;
end;

procedure TStateTransitionNetwork.SetState(Index: Integer;
  const Value: TState);
begin
  if (Index>fStateCount-1) or (Index<0) then
  begin
    raise EOutOfRange.Create('Index ' + IntToStr(Index) + ' out of range');
  end;
  fStates[Index] := Value;
end;

end.
