unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, xmldom, XMLIntf, StdCtrls, msxmldom, XMLDoc, CTT, ComCtrls,
  Menus, ExtCtrls, Buttons, ImgList, ETS, Grids, BaseGrid, AdvGrid, Prototype,
  ScktComp, DataTable ;

type
  TfrmMain = class(TForm)
    xmlImportedCTT: TXMLDocument;
    gbCTT: TGroupBox;
    dlgOpenCTT: TOpenDialog;
    tvCTT: TTreeView;
    pnlToolbar: TPanel;
    mnuMainMenu: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    PrintSetup1: TMenuItem;
    Print1: TMenuItem;
    N2: TMenuItem;
    SaveAs1: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    New1: TMenuItem;
    bttnImportCTT: TSpeedButton;
    imglstTaskCategories: TImageList;
    pnlSimulators: TPanel;
    splitterModelEnabled: TSplitter;
    sbMain: TStatusBar;
    dlgOpenETS: TOpenDialog;
    sckSimulationServer: TServerSocket;
    gbEnabledTasks: TGroupBox;
    tvEnabledTasks: TTreeView;
    splitterSimulator: TSplitter;
    panelEnabledTaskControls: TPanel;
    bttnReset: TButton;
    panelActivityChain: TPanel;
    lbActivityChain: TListBox;
    lblActivityChain: TLabel;
    lblSimulator: TLabel;
    splitterForm: TSplitter;
    bttnPrototype3: TSpeedButton;
    bttnPrototype2: TSpeedButton;
    bttnPrototype1: TSpeedButton;
    bttnDownText: TSpeedButton;
    bttnUpText: TSpeedButton;
    procedure bttnImportClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure tvEnabledTasksDblClick(Sender: TObject);
    procedure bttnResetClick(Sender: TObject);
    procedure sckSimulationServerClientRead(Sender: TObject; Socket: TCustomWinSocket);
    procedure bttnPrototype1Click(Sender: TObject);
    procedure bttnPrototype2Click(Sender: TObject);
    procedure bttnPrototype3Click(Sender: TObject);
    procedure bttnUpTextClick(Sender: TObject);
    procedure bttnDownTextClick(Sender: TObject);
  private
    { Private declarations }
    fcttRootNode : TCTTNode;
    fstnTransitionNetwork : TStateTransitionNetwork;
    fstateCurState : TState;
    fXMLFileName : String;
    fETSFileName : String;
    prototype1, prototype2, prototype3 : TfrmPrototype;
    procedure UpdateCTTTreeView( CurTNNode : TTreeNode; CurCTTNode : TCTTNode);
    procedure DisplayETS(curState : TState);
    procedure UpdateStatusBar;
    procedure ImportCTT;

  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.dfm}

procedure TfrmMain.bttnImportClick(Sender: TObject);
var
  xmlOptions : TXMLDocument;
begin
  prototype1.Free;
  prototype2.Free;
  prototype3.Free;
  prototype1 := TfrmPrototype.Create(Application);
  prototype2 := TfrmPrototype.Create(Application);
  xmlOptions := TXMLDocument.Create(Self);
  xmlOptions.FileName := 'cttTable.xml';
  xmlOptions.Active := true;
  prototype3 := TfrmDataTable.Create(Application, xmlOptions);
  if dlgOpenCTT.Execute then
  begin
    xmlImportedCTT.FileName := dlgOpenCTT.FileName;
    xmlImportedCTT.Active := true;
    ImportCTT;
    fXMLFileName := dlgOpenCTT.FileName;

    if dlgOpenETS.Execute then
    begin
      fETSFileName := dlgOpenETS.FileName;
      fstnTransitionNetwork := ImportSTN(fETSFileName, fcttRootNode);
      fstateCurState := fstnTransitionNetwork.States[fstnTransitionNetwork.InitialStateIndex];
      DisplayETS(fstateCurState);
    end;
  end;
end;

procedure TfrmMain.ImportCTT;
begin
  fcttRootNode := ImportCTTModel(xmlImportedCTT);
  tvCTT.Items.Clear;
  UpdateCTTTreeView(nil, fcttRootNode);
  UpdateStatusBar;
end;

procedure TfrmMain.UpdateStatusBar;
begin
  sbMain.Panels[0].Text := fXMLFileName;
  sbMain.Panels[1].Text := 'Task Count: ' + IntToStr(fcttRootNode.TaskCount);
end;

procedure TfrmMain.UpdateCTTTreeView( CurTNNode : TTreeNode; CurCTTNode : TCTTNode);
var
  NextTNNode : TTreeNode;
  NextCTTNode : TCTTNode;
begin
  while not (CurCTTNode = nil) do
  begin
    //Add the current node.
    NextTNNode := tvCTT.Items.AddChild(CurTNNode, CurCTTNode.Name);

    //Select the appropriate Icon.
    if CurCTTNode.Category = Abstraction then
    begin
      NextTNNode.ImageIndex := 0;
      NextTNNode.SelectedIndex := 0;
    end
    else if CurCTTNode.Category = App then
    begin
      NextTNNode.ImageIndex := 1;
      NextTNNode.SelectedIndex := 1;
    end
    else if CurCTTNode.Category = Interaction then
    begin
      NextTNNode.ImageIndex := 2;
      NextTNNode.SelectedIndex := 2;
    end
    else
    begin
      NextTNNode.ImageIndex := 3;
      NextTNNode.SelectedIndex := 3;
    end;

    //If the current node as a child, add it (recursively)
    if CurCTTNode.HasChild then
    begin
      NextCTTNode := CurCTTNode.Child;
      UpdateCTTTreeView(NextTNNode, NextCTTNode);
    end;
    CurCTTNode := CurCTTNode.SiblingRight;
  end;
end;

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.DisplayETS(CurState: TState);
var
  i : Integer;
  cttCurCTTNode : TCTTNode;
  tnCurTreeNode : TTreeNode;
  curETS : TEnabledTaskSet;
begin
  tvEnabledTasks.Items.Clear;
  if CurState.ClassNameIs('TAcceptState') then
  begin
    tnCurTreeNode := tvEnabledTasks.Items.Add(nil, 'Simulation Completed');
    tnCurTreeNode.ImageIndex := -1;
    tnCurTreeNode.SelectedIndex := -1;
    Exit;
  end;
  curETS := CurState.EnabledTaskSet;
  for i:=0 to curETS.Count-1 do
  begin
    cttCurCTTNode := curETS[i];
    tnCurTreeNode := tvEnabledTasks.Items.Add(nil,cttCurCTTNode.Name);
    //Select the appropriate Icon.
    if cttCurCTTNode.Category = Abstraction then
    begin
      tnCurTreeNode.ImageIndex := 0;
      tnCurTreeNode.SelectedIndex := 0;
    end
    else if cttCurCTTNode.Category = App then
    begin
      tnCurTreeNode.ImageIndex := 1;
      tnCurTreeNode.SelectedIndex := 1;
    end
    else if cttCurCTTNode.Category = Interaction then
    begin
      tnCurTreeNode.ImageIndex := 2;
      tnCurTreeNode.SelectedIndex := 2;
    end
    else
    begin
      tnCurTreeNode.ImageIndex := 3;
      tnCurTreeNode.SelectedIndex := 3;
    end;
  end;
end;

procedure TfrmMain.tvEnabledTasksDblClick(Sender: TObject);
var
  tnSelectedNode : TTreeNode;
  cttPerformedTask : TCTTNode;
begin
  tnSelectedNode := tvEnabledTasks.Selected;
  cttPerformedTask := fcttRootNode.FindChildByName(tnSelectedNode.Text);
  if not (cttPerformedTask = nil) then
  begin
    lbActivityChain.Items.Add(cttPerformedTask.Name);
    fstateCurState := fstnTransitionNetwork.NextState(fstateCurState, cttPerformedTask);
    cttPerformedTask.Performed := true;
    DisplayETS(fstateCurState);
  end;
end;

procedure TfrmMain.bttnResetClick(Sender: TObject);
begin
  fstateCurState := fstnTransitionNetwork.States[fstnTransitionNetwork.InitialStateIndex];
  DisplayETS(fstateCurState);
  lbActivityChain.Items.Clear;
end;

procedure TfrmMain.sckSimulationServerClientRead(Sender: TObject;
  Socket: TCustomWinSocket);
var
  cttPerformedTask : TCTTNode;
  i:integer;
  sRec : string;
begin
  for i := 0 to sckSimulationServer.Socket.ActiveConnections-1 do
  begin
    with sckSimulationServer.Socket.Connections[i] do
    begin
      sRec := ReceiveText;
      if sRec <> '' then
      begin
        cttPerformedTask := fcttRootNode.FindChildByName(sRec);
        if not (cttPerformedTask = nil) then
        begin
          lbActivityChain.Items.Add(cttPerformedTask.Name);
          fstateCurState := fstnTransitionNetwork.NextState(fstateCurState, cttPerformedTask);
          cttPerformedTask.Performed := true;
          DisplayETS(fstateCurState);
        end;
      end;
    end;
  end;
end;

procedure TfrmMain.bttnPrototype1Click(Sender: TObject);
begin
  if not (prototype1 = nil) then
    prototype1.Enabled := not (prototype1.Enabled)
end;

procedure TfrmMain.bttnPrototype2Click(Sender: TObject);
begin
  if not (prototype2 = nil) then
    prototype2.Enabled := not (prototype2.Enabled)
end;

procedure TfrmMain.bttnPrototype3Click(Sender: TObject);
begin
  if not (prototype3 = nil) then
    prototype3.Enabled := not (prototype3.Enabled)
end;

procedure TfrmMain.bttnUpTextClick(Sender: TObject);
begin
  Self.Font.Size := Self.Font.Size + 1;
end;

procedure TfrmMain.bttnDownTextClick(Sender: TObject);
begin
  Self.Font.Size := Self.Font.Size - 1;
end;

end.
