/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;


import java.util.Enumeration;


public class CCTTPriority{
	int $dummyCount = 1;
	
	public CCTTPriority(){
	}


	public CCTTStep getPriorityTree(CCTTStep in){
		return modifyToPriority((CCTTStep)in.clone());
	}

	
	/**
	 * Recursive function that changes the Original Tree into a Priority Tree
	 * @param step Subtree to modify
	 * @return the priority tree
	 */
	private CCTTStep modifyToPriority( CCTTStep step ){
		while( heterogenuousChilds( step ) ){
			insertAbstractTask( step, getHighestPriority( step ) );
		}
		//recursion on children
		Enumeration children = step.getChildren();
		while( children.hasMoreElements() ){
			CCTTStep child = (CCTTStep) children.nextElement();
			if( !child.isLeaf() ){
				modifyToPriority( child );
			}
		}
		return step;
	}
	/**
	 * Returns the Temporal Relation with the highest priority of it's children
	 * @param step The step to examine
	 * @return  the type of the Temporal Relation with the highest priority of it's children
	 */
	private String getHighestPriority( CCTTStep step ){
		Enumeration children = step.getChildren();
		if( step.getChild(0) != null ){
			TemporalRelation highest = step.getChild(0).getNext();
		
			while( children.hasMoreElements() ){
				CCTTStep child = (CCTTStep) children.nextElement();
				if( child.getNext() != null && child.getNext().priorityIsHigher(highest) ){
					highest = child.getNext();
				}
			}
			return highest.getType();
		}
		return null;
	}

	/**
	 * Tests if the step has children with the same temporal oprator
	 * @param step The step to test
	 */
	private boolean heterogenuousChilds( CCTTStep step ){
		//homogenuous step has got 2 or less children
		if( step.getNrOfChildren() < 3 ){
			return false;
		}else{
			Enumeration children = step.getChildren();
			while( children.hasMoreElements() ){
				CCTTStep child = (CCTTStep) children.nextElement();
				//heterogenuous if a child has a previous temporal operator that is different from the next
				if( child.getNext() != null && child.getPrevious() != null && 
					!child.getPrevious().priorityEquals(child.getNext()) )
						return true;
			}
		}
		return false;
	}
							
	/**
	 * Inserts a new Abstract Task for further decomposition
	 * @param step the Parent of the tasks
	 * @param type the type of the Temporal Operator between the tasks to decompose
	 */
	private void insertAbstractTask( CCTTStep step, String type ){
		//locate type operator and decompose the couple from the other siblings
		CCTTStep first  = step.getChild(0);
		while( !first.getNext().getType().equals( type ) ){
			first = first.getRightSibling();
		}
		//we have located the first element to be decomposed
		CCTTStep next = first.getRightSibling();
		while( next.getNext() != null && next.getNext().priorityEquals(first.getNext()) ){
			next = next.getRightSibling();
		}
		CCTTStep second = next;
		
		//introduce new abstract task
		CCTTStep newAbstract = new AbstractTaskStep( "Dummy_"+($dummyCount++));
		newAbstract.setParent( step );

		if( first.getPrevious() != null ){
			//modify temporal
			TemporalRelation tr = first.getPrevious();
			tr.setRightTask( newAbstract );
			newAbstract.attachPrevious( tr );
		}else{
			newAbstract.attachPrevious( null );
		}
		
		if( second.getNext() != null ){
			//modify temporal
			TemporalRelation tr = second.getNext();
			tr.setLeftTask( newAbstract );
			newAbstract.attachNext( tr );
		}else{
			newAbstract.attachNext( null );
			//newAbstract.setRightSibling( null );
		}
		//cut first and second from their siblings and old parent and attach to their new parent
		first.attachPrevious( null );
		//first.setLeftSibling( null );
		second.attachNext( null );
		//second.setRightSibling( null );
		
		step.insertChildAt(step.indexOfChild(first),  newAbstract);
		for(CCTTStep iter_step = first; iter_step != second; iter_step=iter_step.getRightSibling()){
			step.removeChild(iter_step);
			iter_step.setParent(newAbstract);
			newAbstract.addChild(iter_step);
		}
		if(first!=second){
			step.removeChild(second);
			second.setParent(newAbstract);
			newAbstract.addChild(second);
		}
	}

}
