/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
package edm.navigation.concurtasktree;

import java.io.OutputStream;
import java.io.DataOutputStream;

import java.util.Vector;
import java.util.Enumeration;

import edm.navigation.Step;
import edm.navigation.Visitor;
import edm.navigation.UnsupportedElementException;

/**
  * Describes a Task in the ConcurTaskTree model
  * @author <a href="mailto:kris.luyten@luc.ac.be">Kris Luyten</a>, Tim Clerckx
  **/
public abstract class CCTTStep extends Step implements Cloneable{
	private TemporalRelation $previousTR = null;
	private TemporalRelation $nextTR = null;
	private Vector $children = new Vector();
	private CCTTStep $parent=null;
	private int $taskDepth = -1;
	private CCTTStep $leftSibling = null;
	private CCTTStep $rightSibling = null;

	private boolean $iterative=false;
	private boolean $optional=false;
	private boolean $cooperation=false;
	private String $type;
	private String $description;
	private String $timePerfMin;
	private String $timePerfMax;
	private String $timePerfAverage;

	private Vector $platforms = new Vector();
	private Vector $taskObjects = new Vector();
        
   private CCTTStep $searchedTask = null;
   private String $uiPath = null;

	private EnabledTaskSet $next = null;
	public void setNextETS(EnabledTaskSet ets) {
		$next = ets;
	}

	public EnabledTaskSet getNextETS() {
		return $next;
	}



	/**
	 * A subclass will provide the information about the category
	 **/
	public abstract String getCategory();

	
	public CCTTStep(){
		super();
	}

	public CCTTStep(String id){
		super(id);
	}

	public CCTTStep(String id, TemporalRelation proceeds){
		super(id);
		attachPrevious(proceeds);
	}

	public CCTTStep(TemporalRelation proceeds){
		super();
		attachPrevious(proceeds);
	}
	/**
	  * Depth in the hierachical Task decomposition.
	  * @param d the current depth of the composition
	  */
	public void setTaskDepth(int d){
		$taskDepth = d;
	}

	/**
	  * Returns the depth in the hierarchical task decomposition.
	  * @return the depth in the hierarchical task decomposition
	  */
	public int getTaskDepth(){
		if(getParent()==null)
			return 0;
		else return 1+getParent().getTaskDepth();
	}

	public void attachNext(TemporalRelation follows){
		$nextTR = follows;
	}

	public TemporalRelation getNext(){
		return $nextTR;
	}

	public void attachPrevious(TemporalRelation proceeds){
		$previousTR = proceeds;
	}

	public TemporalRelation getPrevious(){
		return $previousTR;
	}

	public void addChild(CCTTStep step){
      $children.add(step);
	  step.setParent(this);
	}

	/**
	 * <b>Overwrites</b> the original CCTTStep object at location index with the new object step.
	 * If you do not wan to overwrite the existing object, see {@link #insertChildAt(int,CCTTStep)} 
	 * @param index the index where the child step will be stored
	 * @param step the CCTTStep that will be stored
	 */
	public void addChildAt( int index, CCTTStep step ){
		$children.add( index, step );
	}

	public void removeChild( CCTTStep step ){
		$children.removeElement( step );
	}
	
	public void insertChildAt(int index, CCTTStep step){
		$children.insertElementAt(step, index);
	}

	/**
	 * Substitutes the internal Vector containing all the children with a new, empty Vector. This
	 * to avoid problems when using the cloning
	 **/
	public void removeChildren(){
		$children = new Vector();
	}

	public int indexOfChild( CCTTStep step ){
		return $children.indexOf( step );
	}

	public Enumeration getChildren(){
		return $children.elements();
	}

	public CCTTStep getChild(int i){
		return (CCTTStep)$children.get(i);
	}

	public void write(OutputStream out){
		DataOutputStream dout = new DataOutputStream(out);
	}

	public CCTTStep getParent(){
		return $parent;
	}

	public void setParent(CCTTStep parent){
		$parent = parent;
	}

	/**
	  * Checks whether test is a sibling of this task step
	  * @param test true if test is a sibling of this CCTTStep, false otherwise
	  **/
	public boolean isSibling(CCTTStep test){
		//we can use the Temporal relations, because all
		//siblings have to be conected by one
		if($previousTR != null){
			try{
				if($previousTR.getLeftTask() == test)
					return true;
			}catch(NullPointerException npe1){
			}
		}

		if($nextTR!=null){
			try{
				if($nextTR.getRightTask() == test)
					return true;
			}catch(NullPointerException npe1){
			}
		}
		return false;
	}


	public boolean isLeaf(){
		return ($children.isEmpty());
	}

	public boolean isOuterRight(){
		//return ($leftSibling == null);
		return( $nextTR == null);
	}

	public boolean isOuterLeft(){
		//return ($rightSibling == null);
		return ($previousTR == null);
	}

	public boolean isIterative(){
		return $iterative;
	}

	public void setIterative( boolean iterative ){
		$iterative = iterative;
	}

	public boolean isOptional(){
		return $optional;
	}

	public void setOptional( boolean optional ){
		$optional = optional;
	}

	public boolean isPartOfCooperation(){
		return $cooperation;
	}

	public void setPartOfCooperation( boolean cooperation ){
		$cooperation = cooperation;
	}

	public String getType(){
		return $type;
	}


	public void setType( String type ){
            $type = type;
	}


	public String getDescription(){
		return $description;
	}
	
	public void setDescription( String description ){
		$description = description;
	}

	public Enumeration getPlatforms(){
		return $platforms.elements();
	}
	 
	 public void addPlatform( String platform ){
		 $platforms.add( platform );
	 }
	 
	 public String getTimePerfMax(){
		 return $timePerfMax;
	 }
	 
	 public void setTimePerfMax( String max ){
		 $timePerfMax = max;
	 }
	 
	 public String getTimePerfMin(){
		 return $timePerfMin;
	 }

	 public void setTimePerfMin( String min ){
		 $timePerfMin = min;
	 }

	 public String getTimePerfAverage(){
		 return $timePerfAverage;
	 }

	 public void setTimePerfAverage( String average ){
		 $timePerfAverage = average;
	 }

	 public Enumeration getTaskObjects(){
		 return $taskObjects.elements();
	 }
	 
	 public void addTaskObject( TaskObject to ){
		 $taskObjects.add( to );
	 }

	 public CCTTStep getLeftSibling(){               
		 //return $leftSibling;
		 if($previousTR != null)
			 return (CCTTStep)$previousTR.getLeftTask();
		 else
			 return null;
	 }
	 	 
	 public CCTTStep getRightSibling(){
		 //return $rightSibling;
		 if($nextTR != null)
			 return (CCTTStep)$nextTR.getRightTask();
		 else
			 return null; 
	 }

	public boolean leftOf(CCTTStep other){
		try{
			if(getRightSibling().isChild(other))
				return true;
			else
				return getRightSibling().leftOf(other);
		}catch(NullPointerException npe){
			return false;
		}
	}

	/**
	 * Checks whether this step or one of the predecessors of this step is involved in a temporal relation of type trType with ancestor
	 * @param trType the type of temporal relation, as defined in {@link edm.navigation.concurtasktree.TemporalRelation TemporalRelation}.
	 * @param @ancestor a candidate ``sibling ancestor''
	 * @return true if this step or one of the predecessors of this step is involved in a temporal relation of type trType with ancestor,
	 * false otherwise.
	 **/
	public boolean ancestorRelation(String trType, CCTTStep ancestor){
		CCTTStep parent = getParent();
		if(parent == null)
			return false;
		
		if(parent.isSibling(ancestor)){
			TemporalRelation tr;
			if((tr = parent.getNext())!=null)
				if(tr.getType().equals(trType))
					return true;

			if((tr = parent.getPrevious())!=null)
				if(tr.getType().equals(trType))
					return true;	
		}
			
		//If the parent is not null, and the two previous tests failed:
		return parent.ancestorRelation(trType, ancestor);
	}

	/**
	 * Returns the first TemporalRelation of type strType in which an ancestor of
	 * this step is involved.
	 * @param trType the type of temporal relation, as defined in {@link edm.navigation.concurtasktree.TemporalRelation TemporalRelation}.
	 * @return the first TemporalRelation of type strType in which an ancestor of this step is involved in.
	 **/
	public TemporalRelation firstAncestorRelation(String trType){
		if(getParent()==null)
			return null;
		else if((getParent().getNext()!=null) && getParent().getNext().getType().equals(trType))
			return getParent().getNext();
		else if((getParent().getPrevious()!=null) && getParent().getPrevious().getType().equals(trType))
			return getParent().getPrevious();
		else
			return getParent().firstAncestorRelation(trType);
	}

	public boolean isChild(CCTTStep s){
		for(Enumeration e = getChildren(); e.hasMoreElements();){
			CCTTStep sp = (CCTTStep)e.nextElement();
			if(s.equals(s))
				return true;
			else return sp.isChild(s);
		}
		return false;
	}


	 /**
	  * returns the number of children of this CCTTStep
	  * @return the number of children of this CCTTStep
	  **/
	 public int getNrOfChildren(){
		 try{
			 return $children.size();
		}catch(NullPointerException npe){
			return 0;
		}
	 }
    
	 	 
	 public void setUiPath(String uiPath){
            $uiPath = uiPath;
	 }
         
	 public String getUiPath(){
		 return $uiPath;
	 }
	 
	 public String toString(){
		 //return getName() + "," + getIdentifier() + ":" + getType() + "[" + getFrequency() + "]";
       return getIdentifier();
	 }
        
	 
	 public CCTTStep findTask(CCTTStep rootStep,String identifier) {
		 $searchedTask = null;
		 findTask(rootStep,identifier,false);
		 return $searchedTask;
	 }
         
    
	 private void findTask(CCTTStep stepNode,String identifier,boolean taskFound) {                         
		 if(stepNode.getIdentifier().equals(identifier)){
			 taskFound = true;
			 $searchedTask = stepNode;
		 }else{            
			 Enumeration e = stepNode.getChildren();
			 while((e.hasMoreElements())&&(!taskFound)){            
				 CCTTStep childNode = (CCTTStep)e.nextElement();
				 findTask(childNode,identifier,taskFound);
			 }
		 }                         
	 }
	 
	 public void accept(Visitor v){
		 try{
			 v.visitStep(this);
		 }catch(UnsupportedElementException uee){
		 }
		 for(Enumeration e = getChildren(); e.hasMoreElements() ;){
				 ((CCTTStep)e.nextElement()).accept(v);
		 }
		 
	 }

	 public void acceptPrint(PrintVisitor pv){
		 pv.start();
		 try{
			 pv.visitStep(this);
		 }catch(UnsupportedElementException uee){
		 }
		 pv.end();
	 }        

	/**
	 * Clones the Subtree , with as root this CCTTStep
	 *  @return The root of the cloned tree
	 */
	 public Object clone(){
		 CCTTStep thisStep = null;
		 try{
			 thisStep = (CCTTStep)super.clone();
			 //empty the $children Vector, because this avoids a lot of mess!!!
			 thisStep.removeChildren();
			 TemporalRelation tr = null;
			 CCTTStep prevChild = null, nextChild = null, clonedNextChild = null, clonedPrevChild = null;
			 int i = 0;
			 while(i<getNrOfChildren()){
				 prevChild = nextChild;
				 clonedPrevChild = clonedNextChild;
				 nextChild = getChild(i);
				 clonedNextChild = (CCTTStep)nextChild.clone();
				 thisStep.addChildAt(i, clonedNextChild);
				 clonedNextChild.setParent(thisStep);
				 i++;
				 if(prevChild!=null){
					 tr = (TemporalRelation)prevChild.getNext().clone();
					 tr.setLeftTask(clonedPrevChild);
					 tr.setRightTask(clonedNextChild);
					 clonedPrevChild.attachNext(tr);
					 clonedNextChild.attachPrevious(tr);
				 }else{
					 clonedNextChild.attachPrevious(null);
				 }
			 }
		 }catch(CloneNotSupportedException e){
		 }
		 return thisStep;
	 }                  
}
