/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

import java.io.OutputStream;

import java.util.Enumeration;

import edm.utils.Tag;

import edm.navigation.Step;
import edm.navigation.UnsupportedElementException;

/**
 * Prints an Enabled Task Set as XML
 * @author <a href="kris.luyten@luc.ac.be">Kris Luyten</a>
 */
public class PrintETSXMLVisitor extends PrintVisitor{
	
	public static String PTSDESCRIPTION = "PtsDescription";
	public static String CTTFILE="CttFile";
	public static String ETSLIST="ETSlist";
	public static String NUMBER="number";
	public static String TASK = "task";
	public static String ETS = "ETS";
	public static String NAME = "name";
	
	private int $etsCounter=0;
	
	
	private Tag $rootTag,$listTag,$cttPartTag;


	public PrintETSXMLVisitor(OutputStream os){
		super(os);
	}


	public void visitStep(Step element) throws UnsupportedElementException{

		try{
			CCTTStep st = (CCTTStep)element;
			print(st);
		}catch(ClassCastException cce){
			throw new UnsupportedElementException(element);			
		}

	}
	
	public void visitStep(EnabledTaskSet ets) throws UnsupportedElementException{
				print(ets);
	}
	

	public void start(){
		getPrintStream().println("<?xml version=\"1.0\"?>");
		$rootTag = new Tag(PTSDESCRIPTION);
		$cttPartTag = new Tag(CTTFILE);
		$listTag = new Tag(ETSLIST);
		getPrintStream().println($rootTag.open());
		getPrintStream().println($cttPartTag.open());
		getPrintStream().println($listTag.open());
	}

	public void end(){
		getPrintStream().println($listTag.close());
		getPrintStream().println($cttPartTag.close());
		getPrintStream().println($rootTag.close());
	}

	protected void print(CCTTStep step){
		
	}
	
	protected void print(EnabledTaskSet ets){
		Tag etsT =  new Tag(ETS);
		etsT.addAttribute(NUMBER, ""+ (++$etsCounter));
		getPrintStream().println("\t" + etsT.open());
		for(Enumeration e=ets.getEnabledTaskSet(); e.hasMoreElements() ; ){
			CCTTStep t = ((EnabledTaskSetElement)e.nextElement()).getStep();
			Tag taskT = new Tag(TASK);
			taskT.addAttribute(NAME, t.getIdentifier());
			getPrintStream().println("\t\t" + taskT.single());			
		}
		getPrintStream().println("\t" + etsT.close());
		
	}

}



