/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

import java.io.OutputStream;

import java.util.Enumeration;

import edm.navigation.Step;
import edm.navigation.UnsupportedElementException;

import edm.utils.Tag;

/**
 * Prints the ConcurTaskTree in the Graph Exchange Language (http://www.gupro.de/GXL/)
 * @author <a href="kris.luyten@luc.ac.be">Kris Luyten</a>
 */
public class PrintGXLVisitor extends PrintVisitor{
	public static String GXL = "gxl";
	public static String NS_XLINK="xmlns:xlink";
	public static String XLINK_url="http://www.w3.org/1999/xlink";
	public static String HREF_XLINK="xlink:href";
	public static String GRAPH = "graph";
	public static String ID = "id";
	public static String NODE = "node";
	public static String TYPE = "type";
	public static String ATTRIBUTE = "attr";
	public static String NAME = "name";
	public static String STRING = "string";
	public static String BOOLEAN = "bool";
	public static String TO = "to";
	public static String FROM = "from";
	public static String EDGE = "edge";
	

	private static int edge_counter = 0;

	
	private static transient Tag tRoot = new Tag(GXL);
	private transient Tag tgraph = new Tag(GRAPH);


	public PrintGXLVisitor(OutputStream os){
		super(os);
	}


	public void visitStep(Step element) throws UnsupportedElementException{
		CCTTStep step = (CCTTStep)element;
		if(step.getParent() == null){
//			tgraph.addAttribute(ID, step.getIdentifier());
			getPrintStream().println(tgraph.open());
		}

		try{
			CCTTStep st = (CCTTStep)element;
			print(st);
		}catch(ClassCastException cce){
			throw new UnsupportedElementException(element);			
		}

		try{
			CCTTStep st = (CCTTStep)element;
			printRelations(st);
		}catch(ClassCastException cce){
			throw new UnsupportedElementException(element);			
		}

		if(step.getParent() == null){
			getPrintStream().println(tgraph.close());
		}

	}

	public void start(){
		getPrintStream().println("<?xml version=\"1.0\"?>");
		getPrintStream().println("<!DOCTYPE gxl SYSTEM \"http://www.gupro.de/GXL/gxl-1.0.dtd\">" );
		tRoot.addAttribute(NS_XLINK, XLINK_url);
		getPrintStream().println(tRoot.open());
	}

	public void end(){
		getPrintStream().println(tRoot.close());
	}

	protected void print(CCTTStep step){
			
		Tag tag = new Tag(NODE);
		tag.addAttribute(ID,step.getIdentifier());		
		getPrintStream().println('\t'+tag.open());
		String whitespace = "\t\t";
		{
		   Tag type = new Tag(TYPE);
			type.addAttribute(HREF_XLINK, step.getCategory());
			getPrintStream().println(whitespace + type.single());
		}

		printAttributes(step);
		
		getPrintStream().println('\t'+tag.close());

		for(Enumeration e = step.getChildren(); e.hasMoreElements() ;){
			 print(((CCTTStep)e.nextElement()));
	   }
		
	}

	private void printAttributes(CCTTStep step){
			
  	   getPrintStream().println("\t\t"+createAttrBoolean(CCTTSAXHandler.ITERATIVE, step.isIterative()));
   	getPrintStream().println("\t\t"+createAttrBoolean(CCTTSAXHandler.OPTIONAL,step.isOptional()));
   	getPrintStream().println("\t\t"+createAttrBoolean(CCTTSAXHandler.PART_OF_COOPERATION,step.isPartOfCooperation()));
		if(step.getFrequency() != null)
		   getPrintStream().println("\t\t"+createAttrString(CCTTSAXHandler.FREQUENCY, ""+ step.getFrequency()));
		

	}

	private String createAttrString(String name, String value){
		Tag tname =  new Tag(ATTRIBUTE);
		tname.addAttribute(NAME, name);
		Tag tvalue = new Tag(STRING);
		return tname.open() + tvalue.open() + value + tvalue.close() + tname.close();		
	}

	private String createAttrBoolean(String name, boolean value){
		Tag tname =  new Tag(ATTRIBUTE);
		tname.addAttribute(NAME, name);
		Tag tvalue = new Tag(BOOLEAN);
		return tname.open() + tvalue.open() + value + tvalue.close() + tname.close();		
	}

	
	private void printRelations(CCTTStep step){
		String whitespace = "\t";

		if(step.getNext() != null){
			Tag sibling_edge = new Tag(EDGE);

			sibling_edge.addAttribute(ID, "edge"+edge_counter++);
			sibling_edge.addAttribute(FROM, step.getIdentifier());
			sibling_edge.addAttribute(TO, step.getNext().getRightTask().getIdentifier());


			getPrintStream().println(whitespace + sibling_edge.open());

			{
				Tag type = new Tag(TYPE);
				type.addAttribute(HREF_XLINK, step.getNext().getType());
				getPrintStream().println('\t' + whitespace + type.single());
			}
			
			getPrintStream().println(whitespace + sibling_edge.close());
		}

		if(step.getNrOfChildren() > 0){
			for(Enumeration e = step.getChildren(); e.hasMoreElements() ;){
				CCTTStep child_step = (CCTTStep)e.nextElement();
				Tag parent_edge = new Tag(EDGE);
				parent_edge.addAttribute(ID, "edge"+edge_counter++);
				parent_edge.addAttribute(FROM, step.getIdentifier());
				parent_edge.addAttribute(TO, child_step.getIdentifier());
				
				getPrintStream().println(whitespace + parent_edge.open());
				
				{
					Tag type = new Tag(TYPE);
					type.addAttribute(HREF_XLINK, "Decomposition");
					getPrintStream().println('\t' + whitespace + type.single());
				}
				
				
				getPrintStream().println(whitespace + parent_edge.close());

				printRelations(child_step);
			}
		}
	}
}



