/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

import java.io.OutputStream;
import java.io.PrintStream;

import edm.navigation.Visitor;
import edm.navigation.Step;
import edm.navigation.UnsupportedElementException;

/**
 * Abstract Visitor class for all kinds of printing
 * @author <a href="kris.luyten@luc.ac.be">Kris Luyten</a>
 */
abstract public class PrintVisitor implements Visitor{
	private OutputStream $os;
	private PrintStream $pos;

	public PrintVisitor(OutputStream os){
		setOutput(os);
	}

	protected OutputStream getOutput(){
		return $os;
	}

	protected PrintStream getPrintStream(){
		return $pos;
	}

	protected void setOutput(OutputStream os){
		$os = os;
		$pos = new PrintStream($os);
	}
	
	/**
	 * Writes the firts part of the output
	 **/
	abstract public void start();

	/**
	 * Writes the last part of the output
	 */
	abstract public void end();

	/**
	 * Visits a Step and writes the appropriate output
	 * @param element the Step that is written to the output
	 * @throws UnsupportedElementException when the object contained in element is not the appropriate subtype
	 * of Step
	 **/
	abstract public void visitStep(Step element) throws UnsupportedElementException;
	
	/**
	 * Prints information of step to the OutputStream os
	 * @param step The CCTTStep that is written to the output
	 * @param os The OutputStream used to write the information
	 **/
	abstract protected void print(CCTTStep step);
	
}



