/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

import java.io.OutputStream;

import java.util.Enumeration;

import edm.navigation.Step;
import edm.navigation.UnsupportedElementException;

import edm.utils.Tag;

/**
 * Prints the ConcurTaskTree in XML
 * @author <a href="kris.luyten@luc.ac.be">Kris Luyten</a>
 */
public class PrintXMLVisitor extends PrintVisitor{
	private static transient Tag tRoot = new Tag(CCTTSAXHandler.TASKMODEL);

	public static transient String NAME = "name";

	public PrintXMLVisitor(OutputStream os){
		super(os);
	}


	public void visitStep(Step element) throws UnsupportedElementException{
		try{
			print((CCTTStep)element);
		}catch(ClassCastException cce){
			throw new UnsupportedElementException(element);
		}
	}

	public void start(){
		getPrintStream().println("<?xml version=\"1.0\"?>");
		//getPrintStream().println("<!DOCTYPE TaskModel PUBLIC \"http://giove.cnuce.cnr.it/CTTDTD.dtd\" []>" );
		getPrintStream().println(tRoot.open());
	}

	public void end(){
		getPrintStream().println(tRoot.close());
	}

	protected void print(CCTTStep step){		
			Tag tag = new Tag(CCTTSAXHandler.TASK);
			setAttributes(tag,step);

			String whitespace = depthString(step);

			getPrintStream().println(whitespace + tag.open());
			printGeneralInfo(step);
			printTimePerf(step);
			printFamily(step);
			printTaskObjects(step);

			if(step.getNrOfChildren()!=0){
				Tag st = new Tag(CCTTSAXHandler.SUBTASK);
				getPrintStream().println(whitespace + st.open());
				for(Enumeration e = step.getChildren(); e.hasMoreElements() ;){
					 print(((CCTTStep)e.nextElement()));
				 }
				getPrintStream().println(whitespace + st.close());
			}

			getPrintStream().println(whitespace + tag.close());
			
	}

	private void setAttributes(Tag tag, CCTTStep step){
		 tag.addAttribute(CCTTSAXHandler.IDENTIFIER, step.getIdentifier());
		 tag.addAttribute(CCTTSAXHandler.CATEGORY, (step.getCategory()==null)?" ":step.getCategory());
		 String s = step.isIterative()?"true":"false";
		 tag.addAttribute(CCTTSAXHandler.ITERATIVE, s);
		 s= step.isOptional()?"true":"false";
		 tag.addAttribute(CCTTSAXHandler.OPTIONAL,s);
		 s= step.isPartOfCooperation()?"true":"false";
		 tag.addAttribute(CCTTSAXHandler.PART_OF_COOPERATION,s);
		 tag.addAttribute(CCTTSAXHandler.FREQUENCY, ""+ step.getFrequency());
	}

	 private void printGeneralInfo(CCTTStep step){
		 String whitespace = depthString(step);
		 Tag name = new Tag(CCTTSAXHandler.NAME);
		 getPrintStream().println(whitespace + name.open() + ((step.getName()==null)?"  ":step.getName()) + name.close());
		 Tag type = new Tag(CCTTSAXHandler.TYPE);
		 getPrintStream().println(whitespace + type.open() + ((step.getType()==null)?"  ":step.getType()) + type.close());
		 Tag description = new Tag(CCTTSAXHandler.DESCRIPTION);
		 getPrintStream().println(whitespace + description.open() + ((step.getDescription()==null)?"  ":step.getDescription()) + description.close());
		 getPrintStream().println(whitespace + "<Precondition>  </Precondition>");
		 if(step.getNext() != null){
			 Tag tempoperator = new Tag(CCTTSAXHandler.TEMPORALOPERATOR);
			 tempoperator.addAttribute(CCTTSAXHandler.TEMP_OP_NAME, step.getNext().getType());
			 getPrintStream().println(whitespace + tempoperator.single());
		 }  
	 }

	 private void printTimePerf(CCTTStep step){
		 String whitespace = depthString(step);
		 Tag timeperformance = new Tag(CCTTSAXHandler.TIMEPERFORMANCE);
		 getPrintStream().println(whitespace + timeperformance.open());
		 Tag max = new Tag(CCTTSAXHandler.MAX);
		 getPrintStream().println(whitespace + '\t' + max.open() + step.getTimePerfMax() + max.close());
		 Tag min = new Tag(CCTTSAXHandler.MIN);
		 getPrintStream().println(whitespace + '\t' + min.open() + step.getTimePerfMin() + min.close());
		 Tag average = new Tag(CCTTSAXHandler.AVERAGE);
		 getPrintStream().println(whitespace + '\t' + average.open() + step.getTimePerfAverage() + average.close());
		 getPrintStream().println(whitespace + timeperformance.close());
	 }


	 private void printFamily(CCTTStep step){
		 String whitespace = depthString(step);

		 //Parent
		 CCTTStep parent = step.getParent();
		 if(parent != null){
			 Tag par = new Tag(CCTTSAXHandler.PARENT);
			 par.addAttribute(NAME,parent.getIdentifier());
			 getPrintStream().println(whitespace + par.single());
		 }
		 
		 //Left Sibling
		 CCTTStep l_sibling = step.getLeftSibling();	 
		 if(l_sibling != null){
			 Tag lsibl = new Tag(CCTTSAXHandler.SIBLINGLEFT);
			 lsibl.addAttribute(NAME,l_sibling.getIdentifier());
			 getPrintStream().println(whitespace + lsibl.single());
		 }
		 
		 //Right Sibling
		 CCTTStep r_sibling = step.getRightSibling();
		 if(r_sibling != null){
			 Tag rsibl = new Tag(CCTTSAXHandler.SIBLINGRIGHT);
			 rsibl.addAttribute(NAME,r_sibling.getIdentifier());
			 getPrintStream().println(whitespace + rsibl.single());
		 }
		 
	 }
	 
	 private void printTaskObjects(CCTTStep step){
		 String whitespace = depthString(step);
		 for (Enumeration e = step.getTaskObjects();e.hasMoreElements();){
			 TaskObject to = (TaskObject)e.nextElement();
			 Tag taskobject = new Tag(CCTTSAXHandler.OBJECT);
			 taskobject.addAttribute(CCTTSAXHandler.TASKOBJ_NAME, to.getName());
			 taskobject.addAttribute(CCTTSAXHandler.TASKOBJ_CLASS, to.getClassName());
			 taskobject.addAttribute(CCTTSAXHandler.TASKOBJ_TYPE, to.getType());
			 taskobject.addAttribute(CCTTSAXHandler.TASKOBJ_ACCESS_MODE, to.getAccessMode());
			 taskobject.addAttribute(CCTTSAXHandler.TASKOBJ_CARDINALITY, to.getCardinality());
			 getPrintStream().println(whitespace + taskobject.open());
			 for(Enumeration platforms = to.getPlatforms();platforms.hasMoreElements();){
				 String platform = (String)platforms.nextElement();
				 Tag pf = new Tag(CCTTSAXHandler.PLATFORM);
				 getPrintStream().println(whitespace + '\t' + pf.open() + platform + pf.close()); 
			 }
			 Tag inputaction = new Tag(CCTTSAXHandler.INPUTACTION);
			 inputaction.addAttribute(CCTTSAXHandler.DESCRIPTION, ((to.getInputDescription()==null)?"null":to.getInputDescription()));
			 inputaction.addAttribute(CCTTSAXHandler.FROM, ((to.getInputFrom()==null)?"null":to.getInputFrom()));
			 getPrintStream().println(whitespace + '\t'+ inputaction.single());
			 Tag outputaction = new Tag(CCTTSAXHandler.OUTPUTACTION);
			 outputaction.addAttribute(CCTTSAXHandler.DESCRIPTION, ((to.getOutputDescription()==null)?"null":to.getOutputDescription()));
			 outputaction.addAttribute(CCTTSAXHandler.TO, ((to.getOutputTo()==null)?"null":to.getOutputTo()));
			 getPrintStream().println(whitespace + '\t' + outputaction.single());
			 getPrintStream().println(whitespace + taskobject.close());
		 }
	 }


	/**
	 * Returns a String of whitespace, according to the level of step in the CTT specification
	 * @param step the CTT Step
	 * @return a String containing whitespaces according to the depth of step in the CTT
	 **/
	private static String depthString(CCTTStep step){
		StringBuffer sb =  new StringBuffer();
		for(int i=0; i< step.getTaskDepth(); i++){
			sb.append('\t');
		}
		return sb.toString();
	}
}



