/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.utils;

//import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;

/**
  Represents a (XML) tag, and gives warnings if open tags were not closed.
  This is a very simple implementation, only meant to quickly build
  a XML-like document for output.
  @author <a href="kris.luyten@luc.ac.be">Kris Luyten</a>
**/
public class Tag{
	private String $tagName;
//	private Hashtable $attributes = null;
	private Vector $attributes = null;
	private boolean $closed=true;

	/**
	 * Constructs a new Tag object with name as an element name
	 * @param name Name of the element
	 **/
	public Tag(String name){
		$tagName=name;
	}

	/**
	  Adds an attribute for this tag
	  @param name the name of the attribute
	  @param value the value of the attribute
	 **/
	public void addAttribute(String name, String value){
		if($attributes == null)
			//$attributes = new Hashtable();
			$attributes = new Vector();

		//$attributes.put(name, value);
		$attributes.add(new String(name + "=\"" + value + "\""));
	}

	/**
	 * Returns all attributes of this tag in a string
	 * @return  all attributes of this tag in a string
	 **/
	private String getAttributesString(){
		if($attributes==null){
			return "";
		}
		StringBuffer list = new StringBuffer();
		//for(Enumeration e = $attributes.keys() ; e.hasMoreElements() ; ){
		for(Enumeration e = $attributes.elements() ; e.hasMoreElements() ; ){
			//String name = (String)e.nextElement();
			//list.append(name).append("=\"").append((String)$attributes.get(name)).append("\" ");
			list.append((String)e.nextElement()).append(' ');
		}
		return " " + list.toString();
	}

	/**
	 * Returns this Tag as an open tag
	 * @return  this Tag as an open tag
	 **/
	public String open(){
		$closed=false;
		return "<" + $tagName + getAttributesString() + ">";		
	}

	/**
	 * Returns this Tag as a close tag
	 * @return  this Tag as a close tag
	 **/
	public String close(){
		$closed=true;
		return "</"+$tagName+">";
	}


	/**
	  Makes sure tags are closed, otherwise it will print an error message.
	  <p>Overriding the finalize method has the advantage this check
	  occurs without the programmer having to do the checks manually</p>
	**/
	protected void finalize() throws Throwable{
		if(!$closed)
			System.err.println("TAG " + $tagName + " IS NOT CLOSED");
	}

	/**
	 * Returns this Tag as a single tag (open and close tags are combined e.g. <blaai/>
	 * @return  this Tag as a single combined tag
	 **/
	public String single(){
			return "<" + $tagName +" "+ getAttributesString() + "/>";
	}
}
