/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/



package edm;


import edm.navigation.concurtasktree.CCTTNavigationReader;
import edm.navigation.concurtasktree.TaskSetGenerator;
import edm.navigation.concurtasktree.IllegalTreeException;
import edm.navigation.concurtasktree.FiniteStateMachine;
import edm.navigation.concurtasktree.UnsupportedTransitionException;
import edm.navigation.concurtasktree.PrintVisitor;
import edm.navigation.concurtasktree.PrintXMLVisitor;
import edm.navigation.concurtasktree.PrintETSXMLVisitor;
import edm.navigation.concurtasktree.PrintGXLVisitor;
import edm.navigation.concurtasktree.CCTTPriority;
import edm.navigation.concurtasktree.EnabledTaskSetCollection;

import edm.parsing.InvalidXMLException;

import java.io.FileNotFoundException;


/**
 * Main class for Tasklib: it provides command-line usage of the library, offering various kinds of processing 
 * @author <a href="mailto:kris.luyten@luc.ac.be">Kris Luyten</a>
 **/
public class TaskLib{
	public final static String VERSION = "0.0.4";

	public final static String[] OPTIONS = {"-priority","-dialog","-ets", "-help","-drawtree","-printXML","-printGXL","-version"};


	final static public void main(String[] args){
		if(args.length < 1){
			printUsage();
			System.exit(0);
		}

		try{			
			if(args[0].equals(OPTIONS[0])){
				if(args.length != 3)
					System.out.println(OPTIONS[0] + " is used together with another switch, use " + OPTIONS[3] + " for more information");
				else if(args[1].equals(OPTIONS[5]))
					priority(args[2], "XML");
			   else if(args[1].equals(OPTIONS[6]))
					priority(args[2], "GXL");
			}else if(args[0].equals(OPTIONS[1])){
				dialog(args[1]);
			}else if(args[0].equals(OPTIONS[2])){
				if(args.length == 3){
					if(args[1].equals(OPTIONS[5]))
						ets(args[2],"XML");
					else
						printUsage();
				}else
					ets(args[1]);
			}else if(args[0].equals(OPTIONS[3])){
				help();
			}else if(args[0].equals(OPTIONS[4])){
				drawtree(args[1]);
			}else if(args[0].equals(OPTIONS[5])){
				if((args.length == 3)&&(args[1].equals(OPTIONS[0])))
					priority(args[2], "XML");
				else
					printXML(args[1]);
			}else if(args[0].equals(OPTIONS[6])){
				if((args.length == 3)&&(args[1].equals(OPTIONS[0])))
					priority(args[2], "GXL");
				else
					printGXL(args[1]);
			}else if(args[0].equals(OPTIONS[7])){
				version();
			}else
				printUsage();
		}catch(ArrayIndexOutOfBoundsException aiobe){
			System.out.println("Incorrect command syntax:");
			printUsage();
		}catch(Exception e){
				System.out.println("/-----------------------STACK------------------------------/");
				e.printStackTrace();
				System.out.println("\\------------------------STACK-----------------------------\\");
				System.out.println("Unrecoverable error");
				System.out.println("Please email the printed stack to the developers");
		}
	}

	public static void priority(String fName, String outputtype){
		PrintVisitor pv;
		if(outputtype.equals("XML"))
			pv = new PrintXMLVisitor(System.out);
		else if(outputtype.equals("GXL"))
			pv = new PrintGXLVisitor(System.out);
		else{
			help();
			return;
		}

		CCTTNavigationReader cnr = new CCTTNavigationReader();		
		try{
			
			cnr.parse(fName);
			(new CCTTPriority()).getPriorityTree(cnr.getCCTTTree()).acceptPrint(pv);
		}catch(InvalidXMLException ixe){
            System.out.println(ixe.toString()); 
      }catch(FileNotFoundException fnfe){
            fnfe.printStackTrace();
      }

	}

	public static void drawtree(String fName){
		System.out.println("Not implemented yet...");
	}

	public static void printXML(String fName){
		printX(new PrintXMLVisitor(System.out), fName);

	}

	public static void printGXL(String fName){
		printX(new PrintGXLVisitor(System.out), fName);
	}
	
	public static void version(){
		System.out.println("TaskLib version " + VERSION);
	}

	private static void printX(PrintVisitor pv, String fName){
		CCTTNavigationReader cnr = new CCTTNavigationReader();		
		try{
			cnr.parse(fName);
			cnr.getCCTTTree().acceptPrint(pv);
		}catch(InvalidXMLException ixe){
            System.out.println(ixe.toString()); 
      }catch(FileNotFoundException fnfe){
            fnfe.printStackTrace();
      }

	}

	public static void ets(String fName, String outputtype){
		CCTTNavigationReader cnr = new CCTTNavigationReader();
		PrintVisitor pv = null;
		if(outputtype.equals("XML"))
			pv = new PrintETSXMLVisitor(System.out);
		try{
			cnr.parse(fName);
			TaskSetGenerator tsg =  new TaskSetGenerator(cnr.getCCTTTree());
			EnabledTaskSetCollection etc = tsg.generateTaskSets();
			etc.acceptPrint(pv);
		}catch(InvalidXMLException ixe){
            System.out.println(ixe.toString()); 
		}catch(IllegalTreeException ite){		
			ite.printStackTrace();
      }catch(FileNotFoundException fnfe){
            fnfe.printStackTrace();
      }
	}

	public static void ets(String fName){
		CCTTNavigationReader cnr = new CCTTNavigationReader();		
		try{
			cnr.parse(fName);
			TaskSetGenerator tsg =  new TaskSetGenerator(cnr.getCCTTTree());
			tsg.generateTaskSets();
			tsg.print(System.out);
		}catch(InvalidXMLException ixe){
            System.out.println(ixe.toString()); 
		}catch(IllegalTreeException ite){		
			ite.printStackTrace();
      }catch(FileNotFoundException fnfe){
            fnfe.printStackTrace();
      }

	}

	public static void help(){
		System.out.println("                                                                              	  ");
		System.out.println("         TaskLib: a Commandline ConcurTaskTree Processing Tool                	  ");
		System.out.println("                                                                             	  ");
		System.out.println("Copyright: Expertise Centre for Digital Media -- Limburgs Universitair Centrum	  ");
		System.out.println("contact: kris.luyten@luc.ac.be                                              	  ");
		System.out.println("web: http://lumumba.luc.ac.be/kris/research/tasklib                              ");
		System.out.println("                                                                          	     ");
		System.out.println("Please email the bugs you find using this tool to us                     	     ");
		System.out.println("                                                                         	     ");
		System.out.println("Options:                                                                     	  ");
		System.out.println("    -ets        Print the Enabled Task Sets                                  	  ");
		System.out.println("    -dialog     Print the Enabled Task Sets and Dialog Model                	  ");
		System.out.println("    -priority   Print the Priority Tree (use with -printXML of -printGXL)	        ");
		System.out.println("    -help       Print this message                                       	     ");
		System.out.println("    -drawtree   Print the tree in ASCII art                                      ");
		System.out.println("    -printXML   Prints the tree as an CTT XML file (experimental!)               ");
		System.out.println("    -printGXL   Outputs the CTT specification using the Graph eXchange Language  (experimental!)");
		System.out.println("    -version    Print version info                                      ");	
		System.out.println("                                                                                 ");
		System.out.println("                                                                                 ");
		System.out.println("                                                                                 ");
		System.out.println("The following options (in any order) can be combined                             ");
      System.out.println("    -ets -printXML                                                               ");
      System.out.println("    -priority -printXML                                                          ");
      System.out.println("    -priority -printGXL                                                          ");
      System.out.println("                                                                                 ");
      System.out.println("                                                                                 ");
		System.out.println("Command syntax:                                                                  ");
		printUsage();
	}


	public static void printUsage(){
		StringBuffer usageString = new StringBuffer();
		usageString.append("Usage: java -jar ");
		usageString.append("tasklib-").append(VERSION).append(".jar");
		usageString.append(" [");
		for(int i=0; i<OPTIONS.length-1; i++){
			usageString.append(OPTIONS[i]).append("|");
		}
		usageString.append(OPTIONS[OPTIONS.length-1]).append("]");
		usageString.append(" <concurtasktreemodel.xml>");
		System.out.println(usageString);
	}


	public static void dialog(String fName){
			CCTTNavigationReader cnr = new CCTTNavigationReader();
		try{
			cnr.parse(fName);
			TaskSetGenerator tsg =  new TaskSetGenerator(cnr.getCCTTTree());
			FiniteStateMachine fsm = new FiniteStateMachine( tsg.getPriorityTree(),  tsg.generateTaskSets().elements() );
         fsm.print( System.out );
		}catch(InvalidXMLException ixe){
            System.out.println(ixe.toString()); 
		}catch(IllegalTreeException ite){		
			ite.printStackTrace();
      }catch(FileNotFoundException fnfe){
         fnfe.printStackTrace();
      }catch(UnsupportedTransitionException ute){
			ute.printStackTrace();
		}

	}
}
