/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation;

import org.xml.sax.SAXException;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;
import org.xml.sax.ErrorHandler; 
import org.xml.sax.helpers.XMLReaderFactory;

import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

import edm.parsing.InvalidXMLException;
import edm.parsing.AbstractSAXHandler;

/**
  Reads a Navigation description using a SAX parser
  @author Kris Luyten
 **/
public abstract class NavigationReader{
	private XMLReader $xr;	

	/**
	 * The SAX Parser class which has to be loaded. In this case the Apache
	 * SAX parser
	 **/
	private static String PNAME="org.apache.xerces.parsers.SAXParser";

	/**
	 * Creates a new NavigationReader: This loads a SAX Parser.
	 **/
	public NavigationReader(){
		try{
			$xr = XMLReaderFactory.createXMLReader(PNAME);
		}catch (SAXException e){
			if(e.getException() != null){
				if(e.getException().getClass().getName().equals("java.lang.ClassNotFoundException")){
					System.out.println(PNAME + " not found!");
					System.out.println("You need an XML parser in the Classpath (Apache Xerces: http://xml.apache.org)");
					System.out.println("or get the binary release."); 
			    }
			}else{
				System.out.println("/-----------------------STACK------------------------------/");
				e.printStackTrace();
				System.out.println("\\------------------------STACK-----------------------------\\");
				System.out.println("Unrecoverable error");
				System.out.println("Please email the printed stack to the developers");
			}
			System.exit(-1);
		}
	}

	/**
	 * Returns the parser used for processing the input document. This
	 * is usefull for switching handlers while processing a document.
	 * @return the parser used for processing the input document. 
	 **/
	public XMLReader getParser(){
		return $xr;
	}

	public void setHandler(AbstractSAXHandler ash){
		$xr.setContentHandler(ash);
	}

	public void setErrorHandler(ErrorHandler eh){
		$xr.setErrorHandler(eh);
	}

	private String $tempName = null;


	/**
	 * Parse an input document, provided by is
	 * @param is The input document
	 * @throws InvalidXMLException When the input document is not a valid XML document
	 **/
	public void parse(InputSource is) throws InvalidXMLException{
		if($xr == null){
			System.err.println("No XML parser available");
			System.exit(-1);
		}
		if(is==null){
			System.err.println("No inputsource available");
			return;
		}
		try{
			if($xr ==null){
				System.err.println("No valid XML parser loaded!");
				System.exit(-1);
			}
			try{
				$xr.parse(is);
			}catch(java.net.UnknownHostException uhe){
				//%^&*&@#$ Microsoft!
				System.out.println($tempName);
				$xr.parse($tempName);
			}
		}catch(SAXException saxe){
			saxe.printStackTrace();
			throw new InvalidXMLException(saxe.toString());
		}catch(IOException ioe){
			ioe.printStackTrace();
			throw new InvalidXMLException(ioe.toString());
		}catch(Exception e){
			e.printStackTrace();
			System.err.println("Unexpected error: please contact software developer");
		}
	}
	/**
	 * Parse an input document, provided by is
	 * @param is The input document
	 * @throws InvalidXMLException When the input document is not a valid XML document
	 **/
	public void parse(InputStream is) throws InvalidXMLException{
		if(is==null){
			System.err.println("Invalid input! (InputStream is null)");
			System.exit(-1);
		}
		parse(new InputSource(is));
	}

	/**
	 * Parse an input document, provided by the File f
	 * @param f The input document
	 * @throws InvalidXMLException When the input document is not a valid XML document
	 * @throws FileNotFoundException When the File f is not found within the filesystem
	 **/
	public void parse(File f) throws InvalidXMLException, FileNotFoundException{
		if(f==null){
			System.err.println("Invalid input! (File is null)");
			System.exit(-1);
		}else{
			parse(new FileInputStream(f));	
		}

			
	}
	
	/**
	 * Parse an input document, provided as a Path description
	 * @param fname The name of the input document
	 * @throws InvalidXMLException When the input document is not a valid XML document
	 * @throws FileNotFoundException When the File f is not found within the filesystem
	 **/
	public void parse(String fname) throws InvalidXMLException, FileNotFoundException{		
		$tempName =  fname;
		parse(new File(fname));
	}
}
