/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation;

import java.util.Vector;
import java.util.Hashtable;

/**
 * Represents a step in a task specification. A step can be a task, subtask, action,...
 * @author <a href="mailto:kris.luyten@luc.ac.be">Kris Luyten</a>
 **/
public abstract class Step implements Cloneable{
	private Vector $nextStep;
	private Step $previousStep;
	private Vector $conditions;
	private String $identifier;
	private String $name;
	private String $frequency;
	private boolean $executing=false;
	
	private Hashtable linkedObjects;


	/**
	  * Step constructor
	  */
	public Step(){
		$conditions=new Vector();
		$nextStep = new Vector();
	}

	/**
	  * Step constructor.
	  * @param id The identifier fore this Step
	  */
	public Step(String id){
		setIdentifier(id);
	}

	/**
	 * Executes this step in the taskmodel. If a UI is
	 * required getInteractionSession can be used
	 **/
	abstract public void execute();

	/**
	 * Accepts a Visitor class (cfr. Design Patterns, Gamma et. al. "Visitor")
	 * @param v the Visitor class
	 * @see edm.navigation.Visitor
	 **/
	abstract public void accept(Visitor v);

	/**
	 * Accepts a PrintVisitor, for various kind of output
	 * @param pv a PrintVisitor for writing the CTT. This can support different formats
	 * @see edm.navigation.concurtasktree.PrintVisitor
	 **/
	abstract public void acceptPrint(edm.navigation.concurtasktree.PrintVisitor pv);

	/**
	 * Returns whether this Step and the otherStep represent the same (sub)task
	 * @param otherStep The other Step
	 * @return true if otherStep and this Step have the same identifier
	 */
	public boolean equals(Step otherStep){
		return getIdentifier().equals(otherStep.getIdentifier());
	}

	public String getIdentifier(){
		return $identifier;
	}

	public void setIdentifier(String id){
		$identifier = id;
	}

	public void setName(String name){
		$name = name;
	}

	public String getName(){
		return $name;
	}


	public void setPreviousStep(Step s){
		$previousStep = s;
	}

	public Step getPreviousStep(){
		return $previousStep;
	}

	public void attachStep(Step p){
		if($nextStep == null)
			$nextStep = new Vector();
		$nextStep.add(p);
	}

	public void setFrequency(String frequency){
		$frequency=frequency;
	}

	public String getFrequency(){
		return $frequency;
	}

	protected void setExecuting(boolean busy){
		$executing = busy;
	}

	public boolean isExecuting(){
		return $executing;
	}


	protected void setCondition(Condition c, boolean value){
	}

	protected void addCondition(Condition cond){
		$conditions.addElement(cond);
	}

	public Object clone() throws CloneNotSupportedException{
		return super.clone();
	}                
}
