/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

import java.io.OutputStream;
import java.io.PrintStream;

import java.util.Vector;
import java.util.Enumeration;


/**
  * Represents an Enabled Task Set
  * @author Tim Clerckx
  **/

public class EnabledTaskSet extends Vector{
	String cn = this.getClass().getName();

	public Enumeration getEnabledTaskSet(){
		return elements();
	}

	/**
	 * Returns whether this is an "intermediate" set: still
	 * containing First or Body predicates, or a completely
	 * calculated set.
	 **/
	public boolean isResolved(){
		for(Enumeration e=getEnabledTaskSet(); e.hasMoreElements();){
		  EnabledTaskSetElement etse = (EnabledTaskSetElement)e.nextElement();
			if( (etse.getType().equals(EnabledTaskSetElement.FIRST)) ||  
				 (etse.getType().equals(EnabledTaskSetElement.BODY)))
				return false;
		}
		return true;
	}

	public void addEnabledTaskSetElement( CCTTStep step, String type ){
		EnabledTaskSetElement etse = new EnabledTaskSetElement( step, type );
		add( etse );
	}	

	public void addEnabledTaskSetElement( EnabledTaskSetElement pEtse ){
		EnabledTaskSetElement etse = new EnabledTaskSetElement( pEtse.getStep(), pEtse.getType() );
		add( etse );
	}

	/**
	 * Returns true if task belongs to this enabled task set.
	 * @param task The task that is verified to be contained in the set.
	 **/
	public boolean contains( CCTTStep task ){
		for( Enumeration e = getEnabledTaskSet(); e.hasMoreElements(); ){
			EnabledTaskSetElement etse = (EnabledTaskSetElement) e.nextElement();
			if( task.equals( etse.getStep() ) )
				return true;
		}
		return false;
	}


	public boolean isEqual( EnabledTaskSet other ){
		for( Enumeration e = elements(); e.hasMoreElements(); ){
			EnabledTaskSetElement etse = (EnabledTaskSetElement) e.nextElement();
			boolean found = false;
			Enumeration othere = other.elements(); 
			while( !found && othere.hasMoreElements() ){
				EnabledTaskSetElement otheretse = (EnabledTaskSetElement) othere.nextElement();
				if( otheretse.getStep() == etse.getStep() ){
					found = true;
				}
			}
			if( !found ){
				return false;
			}
		}
		return true;
	}


	public String toString(){
		StringBuffer s = new StringBuffer();
		if( size() > 0 ){
			EnabledTaskSetElement etse = (EnabledTaskSetElement) elementAt(0);
			if( etse != null && etse.getStep() != null ){
				if( etse.getType() == EnabledTaskSetElement.FIRST ){
					s.append( "{ First("+etse.getStep().getIdentifier()+")" );
				}else if( etse.getType() == EnabledTaskSetElement.BODY ){
					s.append( "{ Body("+etse.getStep().getIdentifier()+")" );
				}else{
					s.append( "{ "+etse.getStep().getIdentifier() );
				}
				
				for( int i=1; i<size(); i++ ){
					etse = (EnabledTaskSetElement) elementAt(i);
					if( etse.getType() == EnabledTaskSetElement.FIRST ){
						s.append( ", First("+etse.getStep().getIdentifier()+")" );
					}else if( ", Body("+etse.getType() == EnabledTaskSetElement.BODY+")" ){
						s.append( ", "+etse.getStep().getIdentifier() );
					}else{
						s.append( ", "+etse.getStep().getIdentifier() );
					}
				}        
				s.append(" } ");
			}
		}
		return s.toString();
	
	}
	
	/**
	 * Prints the Enabled Task Set to a given OutputStream
	 * @param os the OutputStream used for printing the Enabled Task Set
	 **/
	public void print( OutputStream os ){
		PrintStream output = new PrintStream(os);
		output.print(toString());	
	}
}


