/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


package edm.navigation.concurtasktree;

/**
  * Represents an Element in an Enabled Task Set. 
  * This can be one of three things: a regular task that is a leaf, a task with the First predicate,
  * or a task with the Body predicate
  * @author Tim Clerckx, Kris Luyten
  **/
public class EnabledTaskSetElement{
	public final static String FIRST                 = "First";
	public final static String BODY                  = "Body";
	public final static String NONE                  = "None";
	
	private CCTTStep $step;
	private String $type;

	private String $mark;
	
	public EnabledTaskSetElement( CCTTStep step, String type ){
		$step = step;
		$type = type;
	}
	
	public String getType(){
		return $type;
	}

	public String toString(){
		return $step.toString();
	}
	
	public CCTTStep getStep(){
		return $step;
	}
	
	public void setType( String type ){
		$type = type;
	}
	
	public void setStep( CCTTStep step ){
		$step = step;
	}

	/**
	 * Can be used to annotate the element with information usefull during processing
	 * @param marh the information that is added to this element
	 **/
	public void setMark(String mark){
		$mark = mark;
	}

	/**
	 * Gets the information that annotates this element
	 * @return the information that annotates this element
	 **/
	public String getMark(){
		return $mark;
	}
}
