/*
    TaskLib; a Commandline ConcurTaskTree Processing Tool (http://lumumba.luc.ac.be/kris/research/tasklib)
    Copyright (C) 2003  Kris Luyten (kris.luyten@luc.ac.be)
	                     Expertise Centre for Digital Media (http://edm.luc.ac.be)
								Limburgs Universitair Centrum

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of	the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package edm.navigation.concurtasktree;

import java.util.Enumeration;
import java.util.Vector;

import java.io.PrintStream;
import java.io.OutputStream;

import edm.TagMapper;


/**
 * Finite State Machine representing the dynamic behaviour of the User Interface defined by a Task Model.
 * @author Tim Clerckx
 */
public class FiniteStateMachine{	
	
	private Vector $transitions = new Vector();
	private Vector $enabledTaskSets = new Vector();
	private CCTTStep $tree = null;
	private int $start = -1;

	protected static FiniteStateMachine currentFSM = null;
	private int $currentETS = -2;
	private static EnabledTaskSet $cets = null;
	private String $actionTask;

	private int $rrType = -1;

	/**
	 * Default Constructor. To instantiate an Action.
	 */
	public FiniteStateMachine(){
	}
	
	/**
	 * Creates new Finite State Machine. Remark: the steps in the enabled task sets must point to 
	 * the same CCTTStep objects as in the tree.
	 * @param tree The root of the Concurtasktree representing the Task Model
	 */
	public FiniteStateMachine( CCTTStep tree, Enumeration etsCollection ) throws UnsupportedTransitionException {
		$tree = tree;
		FiniteStateMachineConstructor fsmc = new FiniteStateMachineConstructor( $tree, etsCollection );
		Enumeration etsCol = fsmc.getEnabledTaskSets();
		$enabledTaskSets.removeAllElements();
		while( etsCol.hasMoreElements() ){
			EnabledTaskSet ets = (EnabledTaskSet) etsCol.nextElement();
			$enabledTaskSets.addElement( ets );
		}
		Enumeration trans = fsmc.getTransitions();
		$transitions.removeAllElements();
		while( trans.hasMoreElements() ){
			FSMTransition t = (FSMTransition) trans.nextElement();
			$transitions.addElement( t );
		}
		$start = fsmc.getStartTaskSet();
	}


	public CCTTStep getTask( String task ){
		for( Enumeration e = currentFSM.$enabledTaskSets.elements(); e.hasMoreElements(); ){
			EnabledTaskSet ets = (EnabledTaskSet)e.nextElement();
			for( Enumeration e2 = ets.getEnabledTaskSet(); e2.hasMoreElements(); ){
				CCTTStep step = ((EnabledTaskSetElement)e2.nextElement()).getStep();
				if( step.getIdentifier().equals(task) ){
					return step;
				}
			}
		}
		return null;
	}


	public String toString(){
		return TagMapper.TRIGGER;
	}

	public void contractTransitions( /* Option */ ){
	}

	/**
	 * Print the task sets and Finite State Machine on an OutputStream.
	 * @param os The OutputStream to print on.
	 */
	public void print( OutputStream os ){
		PrintStream out = new PrintStream( os );
		out.println();
		out.println("Task specification to Dialog specification");
		out.println();
		out.println("1. Task Sets:");
		out.println("=============");
		printTaskSets( os );
		out.println();
		out.println("2. Start Task Set:");
		out.println("==================");
		printStartTask( os );
		out.println();
		out.println("3. Transitions:");
		out.println("===============");
		printTransitions( os );
	}

	public void printTaskSets( OutputStream os ){
		PrintStream out = new PrintStream( os );
		for( int i = 0; i<$enabledTaskSets.size(); i++ ){
			out.print(" ETS "+i+": ");
			EnabledTaskSet ets = (EnabledTaskSet) $enabledTaskSets.elementAt(i);
			ets.print( os );
			out.println();
		}
	}
	
	public void printStartTask( OutputStream os ){
		PrintStream out = new PrintStream( os );
		out.println( " ETS " + $start );
	}

	public void printTransitions( OutputStream os ){
		PrintStream out = new PrintStream( os );
		for( int i = 0; i<$transitions.size(); i++ ){
			FSMTransition trans = (FSMTransition) $transitions.elementAt(i);
			switch( trans.$type ) {
				case FSMTransition.ENABLING : 
					out.print( " Enabling  transition: " );
					break;
				case FSMTransition.DISABLING: 
					out.print( " Disabling transition: " );
					break;
				case FSMTransition.EXPIRING:
					out.print( " Expiring  transition: " );
					break;
				case FSMTransition.ITERATIVE:
					out.print( " Iterative transition: " );
					break;
			}
			out.print( "ETS "+trans.$from+" -> "+trans.$trigger.getIdentifier()+" -> " );
			if( trans.$to == -1 ){//accept state
				out.println( "Accept State" );
			}else{
				out.println( "ETS "+trans.$to );
			}
		}
	}
}
